/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.explorer.view;

import javax.annotation.Nonnull;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import javax.swing.UIManager;
import javax.swing.ListCellRenderer;
import org.openide.explorer.view.ListView;
import it.tidalwave.netbeans.explorer.view.impl.PatchedListNodeRenderer;

/***********************************************************************************************************************
 *
 * A customisation of ListView which exposes methods for:
 * <ul>
 * <li>changing the colors;</li>
 * <li>setting a custom cell renderer;</li>
 * </ul>
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class EnhancedListView extends ListView
  {
//    private ListCellRenderer listCellRenderer;
    private boolean listInitialized = false;
    
    private boolean rendererPatched = false;

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override
    public void setFont (final @Nonnull Font font)
      {
        super.setFont(font);

        if (list != null)
          {
            list.setFont(font);
          }
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override
    public void setBackground (final @Nonnull Color color)
      {
        super.setBackground(color);

        if (list != null)
          {
            list.setBackground(color);
          }
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override
    public void setForeground (final @Nonnull Color color)
      {
        super.setForeground(color);

        if (list != null)
          {
            list.setForeground(color);
          }
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override
    public void setOpaque (final boolean opaque)
      {
        super.setOpaque(opaque);

        if (list != null)
          {
            list.setOpaque(opaque);
          }
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    public void setCellRenderer (final @Nonnull ListCellRenderer listCellRenderer)
      {
//        this.treeCellRenderer = treeCellRenderer;

        if (list != null)
          {
            list.setCellRenderer(listCellRenderer);
          }
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Nonnull
    public ListCellRenderer getCellRenderer()
      {
        patchRenderer(); // so callers would get the real renderer, eventually patched
        return list.getCellRenderer();
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Override
    public void paint (final @Nonnull Graphics g)
      {
        ensureListIsInitialized();
        super.paint(g);
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    private void ensureListIsInitialized()
      {
        if (!listInitialized)
          {
            list.setOpaque(isOpaque());
            list.setBackground(getBackground());
            list.setForeground(getForeground());
            listInitialized = true;
          }

        patchRenderer();
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    private void patchRenderer()
      {
        if (Boolean.getBoolean("nb.useSwingHtmlRendering") && !rendererPatched)
          {
            rendererPatched = true;
            final PatchedListNodeRenderer nodeRenderer = new PatchedListNodeRenderer(list.getCellRenderer());
            nodeRenderer.setBackgroundSelectionColor(getColorProperty("List.selectionBackground"));
            nodeRenderer.setBackgroundNonSelectionColor(list.getBackground());
            nodeRenderer.setTextSelectionColor(getColorProperty("List.selectionForeground"));
            nodeRenderer.setTextNonSelectionColor(list.getForeground());
            list.setCellRenderer(nodeRenderer);
          }
      }

    /*******************************************************************************************************************
     *
     ******************************************************************************************************************/
    @Nonnull
    private Color getColorProperty (final @Nonnull String propertyName)
      {
        final Object property = getClientProperty(propertyName);

        if (property != null)
          {
            return (Color)property;
          }

        else
          {
            return UIManager.getColor(propertyName);
          }
      }
  }
